==> react-native-ctp-odp-0.1.11.tgz: package/src/OdpViewModule.tsx <==
import React, { forwardRef, useEffect } from 'react';
import { useImperativeHandle, useRef } from 'react';
import {
  requireNativeComponent,
  UIManager,
  findNodeHandle,
  DeviceEventEmitter,
} from 'react-native';
import { ODP_EVENT_LISTENER, ODP_VIEW_REFERENCE } from './CommandTypes';
import { ODP_ON_READY } from './schemas/OdpOnReadyType';
import { ODP_ON_FACILITY_UPDATE } from './schemas/OdpOnFacilityUpdateType';
import { ODP_ON_KEY_VALIDATION } from './schemas/OdpOnKeyValidationType';
import { ODP_ON_NAVIGATION_UPDATE } from './schemas/OdpOnNavigationUpdateType';

const OdpViewManagerCommands = {
  start: 1,
  upload_logs: 2,
};

const startOdp = (viewId: number | null, key: string, host: string) =>
  UIManager.dispatchViewManagerCommand(
    viewId,
    OdpViewManagerCommands.start.toString(),
    [key, host]
  );

const uploadLogs = (viewId: number | null) =>
  UIManager.dispatchViewManagerCommand(
    viewId,
    OdpViewManagerCommands.upload_logs.toString(),
    []
  );

const OdpComponent = requireNativeComponent('OdpViewManager');

const OdpModule = (
  props: {
    onOdpEvent: ODP_EVENT_LISTENER;
  },
  parentRef: React.Ref<unknown> | undefined
) => {
  const nodeRef = useRef(null);

  useEffect(() => {
    console.log('Initializing Event Emitter');
    const eventEmitter = DeviceEventEmitter;
    eventEmitter.addListener('onReady', (event) => {
      props.onOdpEvent.onReady?.(event as ODP_ON_READY);
    });
    eventEmitter.addListener('onKeyValidation', (event) => {
      props.onOdpEvent.onKeyValidation?.(event as ODP_ON_KEY_VALIDATION);
    });
    eventEmitter.addListener('onNavigationUpdate', (event) => {
      props.onOdpEvent.onNavigationUpdate?.(event as ODP_ON_NAVIGATION_UPDATE);
    });
    eventEmitter.addListener('onFacilityUpdate', (event) => {
      props.onOdpEvent.onFacilityUpdate?.(event as ODP_ON_FACILITY_UPDATE);
    });
    return () => {
      eventEmitter.removeAllListeners('onReady');
      eventEmitter.removeAllListeners('onKeyValidation');
      eventEmitter.removeAllListeners('onNavigationUpdate');
      eventEmitter.removeAllListeners('onFacilityUpdate');
    };
  }, []);

  useImperativeHandle(parentRef, () => {
    const nodeHandle = findNodeHandle(nodeRef.current);
    const listeners: ODP_VIEW_REFERENCE = {
      startOdp: (key: string, host: string) => {
        startOdp(nodeHandle, key, host);
      },
      uploadLogs: () => {
        uploadLogs(nodeHandle);
      },
    };
    return listeners;
  });

  return <OdpComponent ref={nodeRef}></OdpComponent>;
};

export default forwardRef(OdpModule);

==> react-native-ctp-odp-0.1.12.tgz: package/src/OdpViewModule.tsx <==
import React, { forwardRef, useEffect } from 'react';
import { useImperativeHandle, useRef } from 'react';
import {
  requireNativeComponent,
  UIManager,
  findNodeHandle,
  DeviceEventEmitter,
  Platform,
} from 'react-native';
import { ODP_EVENT_LISTENER, ODP_VIEW_REFERENCE } from './CommandTypes';
import { ODP_ON_READY } from './schemas/OdpOnReadyType';
import { ODP_ON_FACILITY_UPDATE } from './schemas/OdpOnFacilityUpdateType';
import { ODP_ON_KEY_VALIDATION } from './schemas/OdpOnKeyValidationType';
import { ODP_ON_NAVIGATION_UPDATE } from './schemas/OdpOnNavigationUpdateType';

const OdpViewManagerCommands = {
  start: 1,
  upload_logs: 2,
};

const startOdp = (viewId: number | null, key: string, host: string) =>
  UIManager.dispatchViewManagerCommand(
    viewId,
    OdpViewManagerCommands.start.toString(),
    [key, host]
  );

const uploadLogs = (viewId: number | null) =>
  UIManager.dispatchViewManagerCommand(
    viewId,
    OdpViewManagerCommands.upload_logs.toString(),
    []
  );

const LINKING_ERROR =
  `The package 'react-native-library' doesn't seem to be linked. Make sure: \n\n` +
  Platform.select({ ios: "- You have run 'pod install'\n", default: '' }) +
  '- You rebuilt the app after installing the package\n' +
  '- You are not using Expo Go\n';

const OdpComponent =
  UIManager.getViewManagerConfig('OdpViewManager') != null
    ? requireNativeComponent('OdpViewManager')
    : () => {
        throw new Error(LINKING_ERROR);
      };

const OdpModule = (
  props: {
    onOdpEvent: ODP_EVENT_LISTENER;
  },
  parentRef: React.Ref<unknown> | undefined
) => {
  const nodeRef = useRef(null);

  useEffect(() => {
    console.log('Initializing Event Emitter');
    const eventEmitter = DeviceEventEmitter;
    eventEmitter.addListener('onReady', (event) => {
      props.onOdpEvent.onReady?.(event as ODP_ON_READY);
    });
    eventEmitter.addListener('onKeyValidation', (event) => {
      props.onOdpEvent.onKeyValidation?.(event as ODP_ON_KEY_VALIDATION);
    });
    eventEmitter.addListener('onNavigationUpdate', (event) => {
      props.onOdpEvent.onNavigationUpdate?.(event as ODP_ON_NAVIGATION_UPDATE);
    });
    eventEmitter.addListener('onFacilityUpdate', (event) => {
      props.onOdpEvent.onFacilityUpdate?.(event as ODP_ON_FACILITY_UPDATE);
    });
    return () => {
      eventEmitter.removeAllListeners('onReady');
      eventEmitter.removeAllListeners('onKeyValidation');
      eventEmitter.removeAllListeners('onNavigationUpdate');
      eventEmitter.removeAllListeners('onFacilityUpdate');
    };
  }, []);

  useImperativeHandle(parentRef, () => {
    const nodeHandle = findNodeHandle(nodeRef.current);
    const listeners: ODP_VIEW_REFERENCE = {
      startOdp: (key: string, host: string) => {
        startOdp(nodeHandle, key, host);
      },
      uploadLogs: () => {
        uploadLogs(nodeHandle);
      },
    };
    return listeners;
  });

  // @ts-ignore
  return <OdpComponent ref={nodeRef}></OdpComponent>;
};

export default forwardRef(OdpModule);
